'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _escope = require('escope');

var escope = _interopRequireWildcard(_escope);

var _espree = require('espree');

var _espree2 = _interopRequireDefault(_espree);

var _estraverse = require('estraverse');

var _estraverse2 = _interopRequireDefault(_estraverse);

var _escodegen = require('escodegen');

var _escodegen2 = _interopRequireDefault(_escodegen);

var _inlineSourceMapComment = require('inline-source-map-comment');

var _inlineSourceMapComment2 = _interopRequireDefault(_inlineSourceMapComment);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const espreeConfig = {
    // attach range information to each node
    range: true,
    // attach line/column location information to each node
    loc: true,
    // create a top-level comments array containing all comments
    comment: true,
    // attach comments to the closest relevant node as leadingComments and
    // trailingComments
    attachComment: true,
    // create a top-level tokens array containing all tokens
    tokens: true,
    // specify the language version (3, 5, 6, or 7, default is 5)
    ecmaVersion: 7,
    // specify which type of script you're parsing (script or module, default is script)
    sourceType: 'module',
    // specify additional language features
    ecmaFeatures: {
        modules: true,
        // enable JSX parsing
        jsx: true,
        // enable return in global scope
        globalReturn: true,
        // enable implied strict mode (if ecmaVersion >= 5)
        impliedStrict: true,
        // allow experimental object rest/spread
        experimentalObjectRestSpread: true,
        // allow let and const declarations
        blockBindings: true
    }
};

const namespace = '__NUTRA_MOCK__';

const createIdentifier = id => {
    const identifier = {
        type: 'Identifier',
        name: id
    };
    identifier[namespace] = true;
    return identifier;
};

const setEntryMethod = (entry, id) => ({
    type: 'ExpressionStatement',
    expression: {
        type: 'CallExpression',
        callee: createIdentifier('NutraMock.setEntry'),
        arguments: [{
            type: 'Literal',
            value: entry,
            raw: `"${ entry }"`
        }, {
            type: 'Literal',
            value: id,
            raw: `"${ id }"`
        }, createIdentifier(id)]
    }
});

const mockObject = (entry, id) => ({
    type: 'MemberExpression',
    object: {
        type: 'MemberExpression',
        object: {
            type: 'MemberExpression',
            object: {
                type: 'MemberExpression',
                object: createIdentifier('NutraMock'),
                property: createIdentifier('store'),
                computed: false
            },
            property: {
                type: 'Literal',
                value: entry,
                raw: `"${ entry }"`
            },
            computed: true
        },
        property: {
            type: 'Literal',
            value: id,
            raw: `"${ id }"`
        },
        computed: true
    },
    property: createIdentifier('fake'),
    computed: false
});

const mockObjectExpression = (entry, id) => ({
    type: 'ExpressionStatement',
    expression: mockObject(entry, id)
});

const SourceModifier = (source, filename) => {

    const ast = _espree2.default.parse(source, espreeConfig);
    const entry = _path2.default.join('.', filename.replace(process.cwd(), ''));
    const topImportNodes = {};
    const topConstantNodes = {};

    // collect relevant nodes in the main execution context
    _estraverse2.default.traverse(ast, {
        enter: (node, parent) => {
            if (/FunctionExpression|FunctionDeclaration/.test(node.type)) {
                return _estraverse2.default.VisitorOption.Skip;
            }
        },
        leave: (node, parent) => {
            if (node[namespace]) {
                return;
            }
            if (node.type === 'Identifier' && /Import|Export/.test(parent.type)) {
                if (parent.local && parent.local.name) {
                    parent.local[namespace] = true;
                    if (parent.type.startsWith('Import')) {
                        topImportNodes[parent.local.name] = true;
                    }
                    if (parent.imported && parent.imported.name !== parent.local.name) {
                        parent.imported[namespace] = true;
                    }
                    if (parent.exported && parent.exported.name !== parent.local.name) {
                        parent.exported[namespace] = true;
                    }
                } else {
                    node[namespace] = true;
                }
                return;
            }
            if (node.type === 'VariableDeclarator' && parent.kind === 'const') {
                topConstantNodes[node.id.name] = true;
                node.id[namespace] = true;
            }
        }
    });

    // replace relevant nodes in deep execution contexts
    const topNodes = Object.assign({}, topImportNodes, topConstantNodes);
    const scopeManager = escope.analyze(ast, espreeConfig);
    const currentScopeNodes = {};
    _estraverse2.default.replace(ast, {
        enter: (node, parent) => {
            if (/Function/.test(node.type)) {
                scopeManager.acquire(node).variables.forEach(node => currentScopeNodes[node.name] = true);
            }
        },
        leave: (node, parent) => {
            if (/Function/.test(node.type)) {
                scopeManager.acquire(node).variables.forEach(node => currentScopeNodes[node.name] = false);
            }
            if (node.type === 'Identifier' && !node[namespace] && !currentScopeNodes[node.name] && topNodes[node.name]) {
                if (parent.type === 'ExperimentalSpreadProperty') return;
                if (parent.type === 'MemberExpression' && !parent.computed) return;
                if (parent.type === 'FunctionDeclaration') return;
                if (parent.type === 'VariableDeclarator') return;
                if (parent.type === 'Property') {
                    if (parent.shorthand) {
                        parent.shorthand = false;
                        parent.value = mockObject(entry, node.name);
                    }
                    return;
                }
                return mockObject(entry, node.name);
            }
        }
    });

    // get index of the last import node
    let afterImportNodeIndex = 0;
    for (let i = 0; i < ast.body.length; i++) {
        if (!ast.body[i].type.startsWith('Import')) {
            afterImportNodeIndex = i;
            break;
        }
    }

    // build the import mock registrations
    const importSetEntryMethods = [];
    Object.keys(topImportNodes).forEach(id => importSetEntryMethods.push(setEntryMethod(entry, id)));

    // insert the import mock registrations after the last import node
    let spliceArgs = [afterImportNodeIndex, 0].concat(importSetEntryMethods);
    Array.prototype.splice.apply(ast.body, spliceArgs);

    // insert the constant mock registrations after each constant node
    for (let node, i = afterImportNodeIndex * 2; i < ast.body.length; i++) {
        node = ast.body[i];
        if (node.type === 'ExportNamedDeclaration') {
            if (node.declaration && typeof node.declaration === 'object') {
                node = node.declaration;
            }
        }
        if (node.type !== 'VariableDeclaration' || node.kind !== 'const') {
            continue;
        }
        if (node.declarations && typeof node.declarations[0] === 'object') {
            node = node.declarations[0];
        }
        if (node.type === 'VariableDeclarator' && topConstantNodes[node.id.name]) {
            ast.body.splice(++i, 0, setEntryMethod(entry, node.id.name));
        }
    }

    // reconstruct source
    const modifiedSource = _escodegen2.default.generate(ast, {
        sourceMap: filename,
        sourceMapWithCode: true
    });

    return {
        code: modifiedSource.code,
        map: modifiedSource.map
    };
};

exports.default = SourceModifier;